import { __awaiter, __decorate } from "tslib";
import { customElement, attr, observable } from '@microsoft/fast-element';
import { SystemHealthAlertsBase } from '../foundation-system-health-alerts-base';
import { logger } from '../utils';
import { SYSTEM_HEALTH_ALERTS_SERVER_LOGGER_PREFIX as LOGGER_PREFIX, SYSTEM_HEALTH_ALERTS_SERVER_CLOSE_TIMEOUT as CLOSE_TIMEOUT, SYSTEM_HEALTH_ALERTS_SERVER_EVENTS_NAMES as EVENTS_NAMES, SYSTEM_HEALTH_ALERTS_SERVER_STATUSES as STATUS_TYPE, SYSTEM_HEALTH_ALERTS_SERVER_DEFAULT_STATUS as STATUS_DEFAULT_TYPE, } from './main.config';
import { foundationSystemHealthAlertsServer as styles } from './main.styles';
import { getPrefixedSystemHealthAlertsServer as getTemplate } from './main.template';
const EVENT_BY_STATUS_TYPE = {
    [STATUS_TYPE['UP']]: EVENTS_NAMES.UP,
    [STATUS_TYPE['DOWN']]: EVENTS_NAMES.DOWN,
    [STATUS_TYPE['UNKNOWN']]: EVENTS_NAMES.UNKNOWN,
};
/**
 * The `SystemHealthAlertsServer` component is designed to monitor and display the status of server in real-time.
 *
 * @alpha
 */
export class SystemHealthAlertsServer extends SystemHealthAlertsBase {
    currentServerStatusChanged(oldServerStatus, newServerStatus) {
        if (oldServerStatus !== newServerStatus) {
            this.$emit(EVENTS_NAMES.CHANGED, this.currentServerStatus);
            if (this.displayOnChange) {
                this.display = true;
            }
        }
        this.$emit(EVENT_BY_STATUS_TYPE[newServerStatus]);
    }
    systemStatusChanged(ignore, newStatus) {
        const { socket } = newStatus;
        if (this.currentServerStatus !== STATUS_TYPE.DOWN &&
            socket.isDisconnected &&
            socket.isDisconnectedByServer) {
            this.setServerStatus(STATUS_TYPE.DOWN);
        }
        if (this.currentServerStatus === STATUS_TYPE.DOWN && socket.isConnected) {
            this.setServerStatus(STATUS_TYPE.UP);
        }
    }
    constructor() {
        super();
        this.autoClose = false;
        this.closeTimeout = CLOSE_TIMEOUT;
        this.displayOnChange = true;
        this.titleRestored = 'Server back online';
        this.messageRestored = 'Thank you for your patience while we were restoring service. All functions are now available as normal. Please refresh your page to update';
        this.titleFailed = 'Server outage';
        this.messageFailed = "We're sorry, our servers are experiencing technical difficulties. It is recommended that you save your data before disconnecting from the application. Data is not current and may be incomplete.";
        this.currentServerStatus = STATUS_DEFAULT_TYPE;
        this.display = false;
        this.closeHandler = (event) => {
            if (event === null || event === void 0 ? void 0 : event.target) {
                event.target.$emit(EVENTS_NAMES.CLOSE);
            }
            else {
                this.$emit(EVENTS_NAMES.CLOSE);
            }
        };
        this.rootEventsListeners.push({
            key: EVENTS_NAMES.CLOSE,
            action: () => {
                this.display = false;
            },
        });
    }
    setServerStatus(status) {
        this.currentServerStatus = status;
    }
    getCurrentMessage() {
        if (this.currentServerStatus === STATUS_DEFAULT_TYPE) {
            logger.warn(`${LOGGER_PREFIX}Current server status is not set`);
            return;
        }
        return {
            title: this.currentServerStatus === STATUS_TYPE.UP ? this.titleRestored : this.titleFailed,
            content: this.currentServerStatus === STATUS_TYPE.UP ? this.messageRestored : this.messageFailed,
        };
    }
}
__decorate([
    attr({ attribute: 'auto-close', mode: 'boolean' })
], SystemHealthAlertsServer.prototype, "autoClose", void 0);
__decorate([
    attr({ attribute: 'close-timeout' })
], SystemHealthAlertsServer.prototype, "closeTimeout", void 0);
__decorate([
    attr({ attribute: 'display-onchange', mode: 'boolean' })
], SystemHealthAlertsServer.prototype, "displayOnChange", void 0);
__decorate([
    attr({ attribute: 'title-restored' })
], SystemHealthAlertsServer.prototype, "titleRestored", void 0);
__decorate([
    attr({ attribute: 'message-restored' })
], SystemHealthAlertsServer.prototype, "messageRestored", void 0);
__decorate([
    attr({ attribute: 'title-failed' })
], SystemHealthAlertsServer.prototype, "titleFailed", void 0);
__decorate([
    attr({ attribute: 'message-failed' })
], SystemHealthAlertsServer.prototype, "messageFailed", void 0);
__decorate([
    observable
], SystemHealthAlertsServer.prototype, "currentServerStatus", void 0);
__decorate([
    observable
], SystemHealthAlertsServer.prototype, "display", void 0);
/**
 * ZeroSystemHealthAlertsServer.
 *
 * @remarks
 * A zero version that pre-registers zero components and uses the zero design system.
 *
 * @example
 * ```ts
 * import { ZeroSystemHealthAlertsServer } from '@genesislcap/foundation-system-health-alerts';
 * ...
 * ZeroSystemHealthAlertsServer
 * ```
 *
 * @example Load the micro frontend on-demand
 * ```ts

 *   const { ZeroSystemHealthAlertsServer } = await import('@genesislcap/foundation-system-health-alerts');
 *   return ZeroSystemHealthAlertsServer;
 * },
 * ```
 *
 * @public
 */
let ZeroSystemHealthAlertsServer = class ZeroSystemHealthAlertsServer extends SystemHealthAlertsServer {
    /**
     * @internal
     */
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            yield this.loadRemotes();
        });
    }
    /**
     * @internal
     */
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerCommonZeroComponents } = yield import('../components/zero-components');
            yield registerCommonZeroComponents();
        });
    }
};
ZeroSystemHealthAlertsServer = __decorate([
    customElement({
        name: 'zero-system-health-alerts-server',
        template: getTemplate('zero'),
        styles,
    })
], ZeroSystemHealthAlertsServer);
export { ZeroSystemHealthAlertsServer };
